@@ -580,10 +580,20 @@ static gint _paned_button_press_event(GtkWidget *widget, GdkEventButton *event)
       (event->window == paned->handle) && (event->button == 1))
     {
       paned->in_drag = TRUE;
-      paned->child1_size += event->x - paned->handle_size / 2;
-      paned->child1_size = CLAMP (paned->child1_size, 0,
-                                  widget->allocation.width - gutter_size
-                                  - 2 * GTK_CONTAINER (paned)->border_width);
+      if GTK_IS_VPANED(widget) {
+          paned->child1_size += event->y - paned->handle_size / 2;
+          paned->child1_size = CLAMP (paned->child1_size, 0,
+                                      widget->allocation.height - gutter_size
+                                      - 2 * GTK_CONTAINER (paned)->border_width);
+      }
+      else
+      {
+          paned->child1_size += event->x - paned->handle_size / 2;
+          paned->child1_size = CLAMP (paned->child1_size, 0,
+                                      widget->allocation.width - gutter_size
+                                      - 2 * GTK_CONTAINER (paned)->border_width);
+      }
+                                
     }
 
   return TRUE;
@@ -617,10 +627,17 @@ static gint _paned_motion_notify_event (GtkWidget *widget, GdkEventMotion *event
 
   paned = GTK_PANED (widget);
 
-  if (event->is_hint || event->window != widget->window)
-    gtk_widget_get_pointer(widget, &x, NULL);
-  else
-    x = event->x;
+  if GTK_IS_VPANED(widget) {
+    if (event->is_hint || event->window != widget->window)
+      gtk_widget_get_pointer(widget, NULL, &x);
+    else
+      x = event->y;
+  } else {
+    if (event->is_hint || event->window != widget->window)
+      gtk_widget_get_pointer(widget, &x, NULL);
+    else
+      x = event->x;
+  }
 
   if (paned->in_drag)
     {
