@@ -1,6 +1,118 @@
 #include "misc_functions.h"
 #include "../smooth_rc_style.h"
 
+#ifdef GTK2
+gboolean
+widget_is_a (GtkWidget * widget, gchar * type_name)
+{
+  gboolean result = FALSE;
+
+  if ((widget))
+    {
+      GType tmp = g_type_from_name (type_name);
+      if (tmp)
+	result = g_type_check_instance_is_a ((GTypeInstance *) widget, tmp);
+    }
+
+  return result;
+}
+#endif
+
+#ifdef GTK1
+gboolean
+widget_is_a (GtkWidget * widget, gchar * type_name)
+{
+  gboolean result = FALSE;
+
+  if ((widget))
+    {
+      GtkType tmp = gtk_type_from_name (type_name);
+      if (tmp)
+	result = gtk_type_is_a(widget, tmp);
+    }
+
+  return result;
+}
+#endif
+
+gboolean
+is_toolbar_item (GtkWidget * widget)
+{
+  gboolean result = FALSE;
+
+  if ((widget) && (widget->parent))
+    if ((IS_BONOBO_TOOLBAR (widget->parent))
+	|| (IS_BONOBO_DOCK_ITEM (widget->parent))
+	|| (IS_EGG_TOOLBAR (widget->parent))
+	|| (GTK_IS_TOOLBAR (widget->parent))
+	|| (GTK_IS_HANDLE_BOX (widget->parent)))
+      result = TRUE;
+    else
+      result = is_toolbar_item (widget->parent);
+
+  return result;
+}
+
+gboolean
+is_panel_widget_item (GtkWidget * widget)
+{
+  gboolean result = FALSE;
+
+  if ((widget) && (widget->parent))
+    {
+      if (IS_PANEL_WIDGET (widget->parent))
+	result = TRUE;
+      else
+	result = is_panel_widget_item (widget->parent);
+    }
+  return result;
+}
+
+gboolean
+is_combo_box_entry (GtkWidget * widget)
+{
+  gboolean result = FALSE;
+
+  if ((widget) && (widget->parent))
+    {
+      if (IS_COMBO_BOX_ENTRY (widget->parent))
+	result = TRUE;
+      else
+	result = is_combo_box_entry (widget->parent);
+    }
+  return result;
+}
+
+gboolean
+is_combo_box (GtkWidget * widget)
+{
+  gboolean result = FALSE;
+
+  if ((widget) && (widget->parent))
+    {
+      if (IS_COMBO_BOX (widget->parent))
+	result = TRUE;
+      else
+	result = is_combo_box (widget->parent);
+    }
+  return result;
+}
+
+gboolean
+is_combo (GtkWidget * widget)
+{
+  gboolean result = FALSE;
+
+  if ((widget) && (widget->parent))
+    {
+      if (IS_COMBO (widget->parent))
+	result = TRUE;
+      else
+	result = is_combo (widget->parent);
+    }
+  return result;
+}
+
 /* Many of the following functions were shamelessly copied
  * from multiple engines : the gtk-2.0 metal engine,
  * the XFCE theme engine(GTK2 port), Xenophilia 0.8,
@@ -400,8 +512,8 @@ reverse_engineer_stepper_box (GtkWidget    *range,
 
   *x = (*x - (box_width - *width) / 2) + 2;
   *y = (*y - (box_height - *height) / 2) + 2;
-  *width = box_width - 3;
-  *height = box_height - 3;
+  *width = box_width - 4;
+  *height = box_height - 4;
 #endif
 
 #if GTK1
@@ -428,21 +540,18 @@ reverse_engineer_spin_button (GtkWidget    *widget,
 			      gint         *height)
 {
 #ifdef GTK2
-  gint size = pango_font_description_get_size (widget->style->font_desc);
+ gint size = pango_font_description_get_size (widget->style->font_desc);
   gint realheight, realwidth;
 
   realwidth = MIN(PANGO_PIXELS (size), 30);
 
-  realwidth -= realwidth % 2; /* force even */
-  
   realwidth -= 2 * xthickness(widget->style);
+  realwidth -= realwidth % 2; /* force even */
   
   realheight = ((widget->requisition.height) - 2 * ythickness(widget->style)) / 2;
       
-  realheight -= 1;
-  realwidth += 1;
-  *x += ((*width - realwidth) / 2);
-  *y += ((*height - realheight) / 2) + (arrow_type==GTK_ARROW_DOWN?1:-1);
+  *x = (*x - (realwidth - *width) / 2);
+  *y = (*y - (realheight - *height - (arrow_type==GTK_ARROW_DOWN?1.5:0.5)) / 2);
   *width = realwidth;
   *height = realheight;
 #endif
@@ -472,15 +581,8 @@ reverse_engineer_arrow_box (GtkWidget    *widget,
   #if GTK2
   else if (DETAIL("menuitem"))
     {
-      *width += 2;
-      *height += 2;
-      *x -= 1;
     }
   else if (DETAIL("arrow")) {
-      *width += 2;
-      *height += 2;
-      *x -= 1;
-      *y -= 1;
   }  
   #endif  
   #if GTK1
@@ -998,6 +1100,157 @@ do_draw_option_bevel(GdkWindow * window,
   }  
 }
 
+static void
+SmoothDrawCleanCheckMark(GdkWindow * window, 
+			 GdkGC * gc, 
+			 gint x, 
+			 gint y, 
+			 gint width, 
+			 gint height)
+{
+        /* Short Diagonal */
+	gdk_draw_line(window, gc, x + 1, y + height - 6, x + 1, y + height - 4);/* Left Line */
+        gdk_draw_line(window, gc, x + 2, y + height - 5, x + 2, y + height - 3);/* Right Line */
+
+        /* Long Diagonal */
+        gdk_draw_line(window, gc, x + 3, y + height - 4, x + width - 2, y + 1);/* Top Line */
+        gdk_draw_line(window, gc, x + 3, y + height - 3, x + width - 2, y + 2);/* Center Line */
+        gdk_draw_line(window, gc, x + 3, y + height - 2, x + width - 2, y + 3);/* Bottom Line */
+}
+
+static void
+SmoothDrawFastCheckMark(GdkWindow * window, 
+			GdkGC * gc, 
+			gint x, 
+			gint y, 
+			gint width, 
+			gint height)
+{
+	gdk_draw_line(window, gc, x, y+height-1, x, y+(height/2)-1);
+        gdk_draw_line(window, gc, x, y+height-1, x, y+(height/2)-1);
+        gdk_draw_line(window, gc, x+1, y+height-1, x+1, y+(height/2)-1);
+        gdk_draw_line(window, gc, x+1, y+height-1, x+width, y - 1);
+        gdk_draw_line(window, gc, x, y+height-1, x+width, y);
+}
+
+static void
+SmoothDrawCrossCheckMark(GdkWindow * window, 
+			 GdkGC * gc, 
+		 	 gint x, 
+			 gint y, 
+			 gint width, 
+			 gint height)
+{
+        if (width >= 16) 
+	{
+    	        GdkGCValues gc_vals;
+   	        GdkGC * line_gc = NULL;
+		gint off_1, off_2, off_3;
+		gint line_width = ceil(((width + 1)/5));
+	
+		if (!(line_width % 2)) 
+			line_width -= 1;
+
+		off_2 = line_width;
+		off_1 = off_2 - 1;
+		off_3 = off_2 + 1;
+
+  	        gdk_gc_get_values(gc, &gc_vals);
+	        gc_vals.line_width = line_width;
+	        gc_vals.cap_style = GDK_CAP_ROUND;
+	        
+                line_gc = gdk_gc_new_with_values(window, &gc_vals,   GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_FUNCTION |
+                                                 GDK_GC_CLIP_MASK | GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN | GDK_GC_LINE_WIDTH | 
+                                                 GDK_GC_LINE_STYLE | GDK_GC_CAP_STYLE);
+
+		/* Backward Diagonal */
+		gdk_draw_line(window, line_gc, x+off_1, y+off_1, x+width-off_2, y+height-off_2);/* Center Line */
+
+		/* Forward Diagonal */
+		gdk_draw_line(window, line_gc, x+off_1, y+height - off_2, x+width-off_2, y+off_1);/* Center Line */
+
+   	        gdk_gc_unref(line_gc);
+	} 
+	else 
+	{
+		/* Backward Diagonal */
+		if (width>=7) 
+			gdk_draw_line(window, gc, x+2, y+1, x+width-2, y+height-3);/* Top Line */
+
+		gdk_draw_line(window, gc, x+1, y+1, x+width-2, y+height-2);/* Center Line */
+
+		if (width>=7) 
+			gdk_draw_line(window, gc, x+1, y+2, x+width-3, y+height-2);/* Bottom Line */
+
+		/* Forward Diagonal */
+		if (width>=7) 
+			gdk_draw_line(window, gc, x+1, y+height-3, x+width-3, y+1);/* Top Line */
+
+		gdk_draw_line(window, gc, x+1, y+height - 2, x+width-2, y+1);/* Center Line */
+
+		if (width>=7) 
+			gdk_draw_line(window, gc, x+2, y+height - 2, x+width-2, y+2);/* Bottom Line */
+	}		
+}
+
+static void
+SmoothDrawBlockCheckMark(GdkWindow * window, 
+			 GdkGC * gc, 
+			 gint x, 
+			 gint y, 
+			 gint width, 
+			 gint height)
+{
+	x += 1;
+	y += 1; 
+	width -= 2;
+        height -= 2; 
+
+        gdk_draw_rectangle(window, gc, TRUE, x, y, width, height);
+}
+
+static void
+SmoothDrawCircleCheckMark(GdkWindow * window, 
+			  GdkGC * gc, 
+			  gint x, 
+			  gint y, 
+			  gint width, 
+			  gint height)
+{
+	x += width / 4;
+	y += height / 4; 
+	width = width / 2 + 1;
+        height = height / 2 + 1; 
+
+        gdk_draw_arc(window, gc, TRUE, x, y, width, height, 0, 360*64); 
+        gdk_draw_arc(window, gc, FALSE, x, y, width, height, 0, 360*64); 
+}
+
+static void
+SmoothDrawDiamondCheckMark(GdkWindow * window, 
+			   GdkGC * gc, 
+			   gint x, 
+			   gint y, 
+			   gint width, 
+			   gint height)
+{
+	GdkPoint points[5];
+		         
+	x += 1;
+	y += 1; 
+	width -= 2;
+        height -= 2; 
+	  
+	points[0].x = x + width/2; points[0].y = y;
+	points[1].x = x + width;   points[1].y = y + height/2;
+	points[2].x = x + width/2; points[2].y = y + height;
+	points[3].x = x;           points[3].y = y + height/2;
+	points[4].x = x + width/2; points[4].y = y;
+	  
+	gdk_draw_polygon(window, gc, TRUE, points, 5);
+	gdk_draw_polygon(window, gc, FALSE, points, 5);
+}
+
 void
 draw_check_mark(GdkWindow * window,
                 GdkGC * gc,
@@ -1008,118 +1261,48 @@ draw_check_mark(GdkWindow * window,
 		gint height,
 		gint check_style)
 {
-   switch (check_style) {
-      case FAST_CHECK : /* most common style */
-        gdk_draw_line(window, gc, x, y+height-1, x, y+(height/2)-1);
-        gdk_draw_line(window, gc, x+1, y+height-1, x+1, y+(height/2)-1);
-        gdk_draw_line(window, gc, x+1, y+height-1, x+width, y - 1);
-        gdk_draw_line(window, gc, x, y+height-1, x+width, y);
-	break;
-      case SLOPPY_CHECK : /* default theme style, not done so just fall through*/
-      case CLEAN_CHECK :/* classic redmond style */
-	x+=1;
-	y+=1; 
-	width-=2;
-        height-=2; 
+	if (area) gdk_gc_set_clip_rectangle (gc, area);
 
-        /* short diagonal */
-	gdk_draw_line(window, gc, x+0, y+height-5, x+0, y+height-2);/* Left Line */
-        gdk_draw_line(window, gc, x+1, y+height-4, x+1, y+height-1);/* Right Line */
-
-        /* Long Diagonal */
-        gdk_draw_line(window, gc, x+2, y+height-3, x+width-1, y+0);/* Top Line */
-        gdk_draw_line(window, gc, x+2, y+height-2, x+width-1, y+1);/* Center Line */
-        gdk_draw_line(window, gc, x+2, y+height-1, x+width-1, y+2);/* Bottom Line */
-	break;
-      case X_CHECK : /* common(default?) style in kde themes */
-        {
-	  GdkGC * line_gc = NULL;
-          if (width>=16) 
-	  {
-  	    GdkGCValues gc_vals;
-	    gint off_1, off_2, off_3;
-            gint line_width = ceil(((width + 1)/5));
-	
-	    if (!(line_width % 2)) line_width -= 1;
-	    gdk_gc_get_values(gc, &gc_vals);
-	    gc_vals.line_width = line_width;
-	    gc_vals.cap_style = GDK_CAP_ROUND;
-
-	    off_2 = gc_vals.line_width;
-	    off_1 = off_2 - 1;
-	    off_3 = off_2 + 1;
-
-              line_gc = gdk_gc_new_with_values(window, &gc_vals,   GDK_GC_FOREGROUND | GDK_GC_BACKGROUND | GDK_GC_FUNCTION |
-              GDK_GC_CLIP_MASK | GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN | GDK_GC_LINE_WIDTH | GDK_GC_LINE_STYLE |
-	      GDK_GC_CAP_STYLE);
-
-  	    /* Backward Diagonal */
-            gdk_draw_line(window, line_gc, x+off_1, y+off_1, x+width-off_2, y+height-off_2);/* Center Line */
-
-            /* Forward Diagonal */
-            gdk_draw_line(window, line_gc, x+off_1, y+height - off_2, x+width-off_2, y+off_1);/* Center Line */
-
-  	    gdk_gc_unref(line_gc);
-	  } else {
-	    /* Backward Diagonal */
-            if (width>=7) 
-	      gdk_draw_line(window, gc, x+2, y+1, x+width-2, y+height-3);/* Top Line */
-            gdk_draw_line(window, gc, x+1, y+1, x+width-2, y+height-2);/* Center Line */
-            if (width>=7) 
-              gdk_draw_line(window, gc, x+1, y+2, x+width-3, y+height-2);/* Bottom Line */
-
-            /* Forward Diagonal */
-            if (width>=7) 
-              gdk_draw_line(window, gc, x+1, y+height-3, x+width-3, y+1);/* Top Line */
-            gdk_draw_line(window, gc, x+1, y+height - 2, x+width-2, y+1);/* Center Line */
-            if (width>=7) 
-              gdk_draw_line(window, gc, x+2, y+height - 2, x+width-2, y+2);/* Bottom Line */
-          }	  
-	}
-	break;	
-      case BLOCK_CHECK : /* square check style, like mist */
-        gdk_draw_rectangle(window, gc, TRUE, x+1, y+1, width-2, height-2); 
-	break;	
-      case CIRCLE_CHECK : /* round check style, like mist, but with circles */
-        gdk_draw_arc(window, gc, TRUE, x + width / 4, y + height / 4, width / 2 + 1, height / 2 + 1, 0, 360*64); 
-        gdk_draw_arc(window, gc, FALSE, x + width / 4, y + height / 4, width / 2 + 1, height / 2 + 1, 0, 360*64); 
-	break;	
-      case DIAMOND_CHECK : /* diamond check style, like mist, but with diamonds */
+	switch (check_style) 
 	{
-	  GdkPoint * points = g_new(GdkPoint, 5);
-          
-	  x+=1;
-	  y+=1; 
-	  width-=2;
-	  height-=2; 
-	  
-	  points[0].x = x + width/2;
-	  points[0].y = y;
+		case CLEAN_CHECK : // classic redmond style
+			SmoothDrawCleanCheckMark(window, gc, x, y, width, height);
+		break;
 
-	  points[1].x = x + width;
-	  points[1].y = y + height/2;
+		case FAST_CHECK : // most common style
+			SmoothDrawFastCheckMark(window, gc, x, y, width, height);
+		break;	
 
-	  points[2].x = x + width/2;
-	  points[2].y = y + height;
-	  
-	  points[3].x = x;
-	  points[3].y = y + height/2;
+		case X_CHECK : // common(default?) style in kde themes 
+			SmoothDrawCrossCheckMark(window, gc, x, y, width, height);
+		break;	
 
-	  points[4].x = x + width/2;
-	  points[4].y = y;
-	  
-	  gdk_draw_polygon(window, gc, TRUE, points, 5);
-	  gdk_draw_polygon(window, gc, FALSE, points, 5);
-	  
-	  g_free(points);
-	}
-	break;	
-      case XPM_CHECK : /* custom check in xpm format */
-	break;	
-      case NO_CHECK :
-      default :
-        return;
-   }   
+		case BLOCK_CHECK : /* square check style, like mist */
+			SmoothDrawBlockCheckMark(window, gc, x, y, width, height);
+		break;	
+
+		case CIRCLE_CHECK  : /* round check style, like mist, but with circles */
+			SmoothDrawCircleCheckMark(window, gc, x, y, width, height);
+		break;	
+
+		case DIAMOND_CHECK  : /* diamond check style, like mist, but with diamonds - needs to follow motif option and do 3d edge to allow being gtk1.2 like*/
+			SmoothDrawDiamondCheckMark(window, gc, x, y, width, height);
+		break;	
+
+		case SLOPPY_CHECK :
+			draw_check_mark(window, gc, area, x, y, width, height, CLEAN_CHECK);
+		break;	
+
+		case XPM_CHECK : /* custom check in xpm format */
+			draw_check_mark(window, gc, area, x, y, width, height, CLEAN_CHECK);
+		break;	
+
+		case NO_CHECK :
+		default :
+			return;
+	}   
+	
+	if (area) gdk_gc_set_clip_rectangle (gc, NULL);
 }
 
 gboolean 
@@ -1134,7 +1317,7 @@ TranslateCheckStyleName (gchar * str, gint *retval)
     *retval = SLOPPY_CHECK;
   else if (is_enum("clean"))
     *retval = CLEAN_CHECK;
-  else if (is_enum("criss_cross") || is_enum("criss-cross") || is_enum("crisscross") || is_enum("x"))
+  else if (is_enum("criss_cross") || is_enum("criss-cross") || is_enum("crisscross") || is_enum("cross") || is_enum("x"))
     *retval = X_CHECK;
   else if (is_enum("block") || is_enum("square") || is_enum("box"))
     *retval = BLOCK_CHECK;
